package net.esper.eql.join;

import net.esper.event.EventBean;
import net.esper.event.EventType;
import net.esper.event.EventPropertyGetter;
import net.esper.event.EventBeanUtility;
import net.esper.eql.join.table.EventTable;
import net.esper.eql.join.table.PropertyIndexedEventTable;
import net.esper.eql.join.table.UnindexedEventTableList;

import java.util.Iterator;
import java.util.Set;
import java.util.Arrays;

/**
 * Index lookup strategy into a poll-based cache result.
 */
public class HistoricalIndexLookupStrategyIndex implements HistoricalIndexLookupStrategy
{
    private final EventPropertyGetter[] propertyGetters;

    /**
     * Ctor.
     * @param eventType - event type to expect for lookup
     * @param properties - key properties
     */
    public HistoricalIndexLookupStrategyIndex(EventType eventType, String[] properties)
    {
        propertyGetters = new EventPropertyGetter[properties.length];
        for (int i = 0; i < properties.length; i++)
        {
            propertyGetters[i] = eventType.getGetter(properties[i]);

            if (propertyGetters[i] == null)
            {
                throw new IllegalArgumentException("Property named '" + properties[i] + "' is invalid for type " + eventType);
            }
        }
    }

    public Iterator<EventBean> lookup(EventBean lookupEvent, EventTable indexTable)
    {
        // The table may not be indexed as the cache may not actively cache, in which case indexing doesn't makes sense
        if (indexTable instanceof PropertyIndexedEventTable)
        {
            PropertyIndexedEventTable index = (PropertyIndexedEventTable) indexTable;
            Object[] keys = getKeys(lookupEvent);

            Set<EventBean> events = index.lookup(keys);
            if (events != null)
            {
                return events.iterator();
            }
            return null;
        }

        return indexTable.iterator();
    }

    private Object[] getKeys(EventBean event)
    {
        return EventBeanUtility.getPropertyArray(event, propertyGetters);
    }
}
